"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const before_1 = require("./before");
describe('before', () => {
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidBefore = (0, before_1.before)({ func: null });
            class T {
            }
            __decorate([
                nonValidBefore,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@before is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should verify before method invocation with the right context when provided as string', () => {
        let counter = 0;
        class T {
            before() {
                expect(this.prop).toBe(3);
                expect(counter).toBe(0);
                counter += 1;
            }
            foo(x) {
                return this.goo(x);
            }
            goo(x) {
                expect(this.prop).toBe(3);
                expect(counter).toBe(1);
            }
        }
        __decorate([
            (0, before_1.before)({
                func: 'before',
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", void 0)
        ], T.prototype, "foo", null);
        const t = new T();
        t.prop = 3;
        const spyGoo = jest.spyOn(T.prototype, 'goo');
        const spyBefore = jest.spyOn(T.prototype, 'before');
        t.foo(1);
        expect(spyGoo).toBeCalledTimes(1);
        expect(spyGoo).toBeCalledWith(1);
        expect(spyBefore).toBeCalledTimes(1);
    });
    it('should verify before method invocation when method is provided', () => {
        let counter = 0;
        const beforeFunc = jest.fn(() => {
            expect(counter).toBe(0);
            counter += 1;
        });
        class T {
            foo(x) {
                return this.goo(x);
            }
            goo(x) {
                expect(counter).toBe(1);
            }
        }
        __decorate([
            (0, before_1.before)({
                func: beforeFunc,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", void 0)
        ], T.prototype, "foo", null);
        const t = new T();
        const spyGoo = jest.spyOn(T.prototype, 'goo');
        t.foo(1);
        expect(spyGoo).toBeCalledTimes(1);
        expect(spyGoo).toBeCalledWith(1);
        expect(beforeFunc.mock.calls.length).toBe(1);
    });
    it('should verify before method invocation when method is provided without waiting for it to be resolved', (done) => {
        let counter = 0;
        const beforeFunc = jest.fn(() => new Promise((resolve) => {
            setTimeout(() => {
                expect(counter).toBe(1);
                resolve(null);
                done();
            }, 0);
        }));
        class T {
            foo(x) {
                return this.goo(x);
            }
            goo(x) {
                expect(counter).toBe(0);
                counter += 1;
            }
        }
        __decorate([
            (0, before_1.before)({
                func: beforeFunc,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", void 0)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo(1);
    });
    it('should verify before method invocation when method is provided with waiting for it to be resolved', (done) => {
        let counter = 0;
        const beforeFunc = jest.fn(() => new Promise((resolve) => {
            setTimeout(() => {
                expect(counter).toBe(0);
                counter += 1;
                resolve(null);
                done();
            }, 10);
        }));
        class T {
            foo(x) {
                return this.goo(x);
            }
            goo(x) {
                expect(counter).toBe(1);
            }
        }
        __decorate([
            (0, before_1.before)({
                func: beforeFunc,
                wait: true,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", void 0)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo(1);
    });
    it('should returned result of method', () => __awaiter(void 0, void 0, void 0, function* () {
        let counter = 0;
        const beforeFunc = jest.fn(() => {
            expect(counter).toBe(0);
            counter += 1;
        });
        class T {
            foo(x) {
                return x;
            }
        }
        __decorate([
            (0, before_1.before)({
                func: beforeFunc,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", Number)
        ], T.prototype, "foo", null);
        const t = new T();
        const value = 42;
        const result = yield t.foo(value);
        expect(result).toBe(value);
    }));
});
//# sourceMappingURL=before.spec.js.map