"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const cancel_previous_1 = require("./cancel-previous");
const canceled_promise_1 = require("./canceled-promise");
describe('cancelPrevious', () => {
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidCancelPrevious = (0, cancel_previous_1.cancelPrevious)();
            class T {
            }
            __decorate([
                nonValidCancelPrevious,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@cancelPrevious is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should cancel prev invocation', (done) => {
        class T {
            foo(x) {
                return new Promise((resolve) => {
                    setTimeout(() => {
                        resolve(x);
                    }, 10);
                });
            }
        }
        __decorate([
            (0, cancel_previous_1.cancelPrevious)(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        let cancelHappened = 0;
        const func = (x) => {
            t.foo(x)
                .then((data) => {
                expect(data).toBe(100);
                expect(cancelHappened).toBe(1);
                done();
            })
                .catch((e) => {
                if (e instanceof canceled_promise_1.CanceledPromise) {
                    expect(e.message).toBe('canceled');
                    cancelHappened += 1;
                    return;
                }
                throw new Error('should\'t get here');
            });
        };
        func(10);
        setTimeout(() => {
            func(100);
        }, 5);
    });
    it('should invoke original method id was resolved before second call', (done) => {
        class T {
            foo(x) {
                return new Promise((resolve) => {
                    setTimeout(() => {
                        resolve(x);
                    }, 10);
                });
            }
        }
        __decorate([
            (0, cancel_previous_1.cancelPrevious)(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        let cancelHappened = 0;
        let round = 1;
        const func = (x) => {
            t.foo(x)
                .then((data) => {
                if (round === 1) {
                    round += 1;
                    expect(data).toBe(10);
                }
                else {
                    expect(data).toBe(100);
                }
                expect(cancelHappened).toBe(0);
                done();
            })
                .catch((e) => {
                if (e instanceof canceled_promise_1.CanceledPromise) {
                    cancelHappened += 1;
                }
                else {
                    throw new Error('should\'t get here');
                }
            });
        };
        func(10);
        setTimeout(() => {
            func(100);
        }, 15);
    });
    it('should invoke rejection if original method got an error', (done) => {
        class T {
            foo(x) {
                return new Promise((resolve, reject) => {
                    setTimeout(() => {
                        reject(new Error('server error'));
                    }, 10);
                });
            }
        }
        __decorate([
            (0, cancel_previous_1.cancelPrevious)(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        const func = (x) => {
            t.foo(x)
                .then(() => {
                throw new Error('should\'t get here');
            })
                .catch((e) => {
                if (e instanceof canceled_promise_1.CanceledPromise) {
                    throw new Error('should\'t get here');
                }
                else {
                    expect(e.message).toBe('server error');
                    done();
                }
            });
        };
        func(10);
    });
});
//# sourceMappingURL=cancel-previous.spec.js.map