"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const task_exec_1 = require("./task-exec");
const test_utils_1 = require("../test-utils");
describe('utils', () => {
    it('should verify task executed in time - A:40, B:20 -> B, A', () => __awaiter(void 0, void 0, void 0, function* () {
        const runner = new task_exec_1.TaskExec();
        let val = '';
        const funA = jest.fn().mockImplementation(() => {
            val += 'A';
        });
        const funB = jest.fn().mockImplementation(() => {
            val += 'B';
        });
        runner.exec(funA, 100);
        yield (0, test_utils_1.sleep)(20);
        expect(funA).not.toBeCalled();
        runner.exec(funB, 40);
        yield (0, test_utils_1.sleep)(20);
        expect(funA).not.toBeCalled();
        expect(funB).not.toBeCalled();
        yield (0, test_utils_1.sleep)(40);
        expect(funA).not.toBeCalled();
        expect(funB).toBeCalledTimes(1);
        yield (0, test_utils_1.sleep)(100);
        expect(funA).toBeCalledTimes(1);
        expect(funB).toBeCalledTimes(1);
        expect(val).toBe('BA');
    }));
    it('should verify task executed in time - A:20, B:40 -> A, B', () => __awaiter(void 0, void 0, void 0, function* () {
        const runner = new task_exec_1.TaskExec();
        let val = '';
        const funA = jest.fn().mockImplementation(() => {
            val += 'A';
        });
        const funB = jest.fn().mockImplementation(() => {
            val += 'B';
        });
        runner.exec(funA, 50);
        runner.exec(funB, 100);
        yield (0, test_utils_1.sleep)(20);
        expect(funA).not.toBeCalled();
        expect(funB).not.toBeCalled();
        yield (0, test_utils_1.sleep)(50);
        expect(funA).toBeCalled();
        expect(funB).not.toBeCalled();
        yield (0, test_utils_1.sleep)(50);
        expect(funA).toBeCalledTimes(1);
        expect(funB).toBeCalledTimes(1);
        expect(val).toBe('AB');
    }));
    it('should verify task executed in time -  - A:20, B:20, C:10 -> A, B, C', () => __awaiter(void 0, void 0, void 0, function* () {
        const runner = new task_exec_1.TaskExec();
        const funA = jest.fn();
        const funB = jest.fn();
        const funC = jest.fn();
        runner.exec(funA, 50);
        runner.exec(funB, 50);
        yield (0, test_utils_1.sleep)(20);
        expect(funA).not.toBeCalled();
        expect(funB).not.toBeCalled();
        runner.exec(funC, 10);
        yield (0, test_utils_1.sleep)(50);
        expect(funA).toBeCalledTimes(1);
        expect(funB).toBeCalledTimes(1);
        expect(funC).toBeCalledTimes(1);
    }));
});
//# sourceMappingURL=task-exec.spec.js.map