"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const memoize_1 = require("./memoize");
const test_utils_1 = require("../common/test-utils");
describe('memozie', () => {
    it('should verify memoize caching original method', (done) => {
        class T {
            foo(x, y) {
                return this.goo(x, y);
            }
            goo(x, y) {
                expect(this.prop).toBe(3);
                return x + y;
            }
        }
        __decorate([
            (0, memoize_1.memoize)(10),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number, Number]),
            __metadata("design:returntype", Number)
        ], T.prototype, "foo", null);
        const t = new T();
        t.prop = 3;
        const spy = jest.spyOn(T.prototype, 'goo');
        const resp1 = t.foo(1, 2);
        const resp2 = t.foo(1, 2);
        const resp4 = t.foo(1, 3);
        expect(spy).toBeCalledTimes(2);
        expect(spy.mock.calls[0][0]).toBe(1);
        expect(spy.mock.calls[0][1]).toBe(2);
        expect(spy.mock.calls[1][0]).toBe(1);
        expect(spy.mock.calls[1][1]).toBe(3);
        setTimeout(() => __awaiter(void 0, void 0, void 0, function* () {
            const resp3 = t.foo(1, 2);
            expect(spy).toBeCalledTimes(3);
            expect(resp1).toBe(3);
            expect(resp2).toBe(3);
            expect(resp3).toBe(3);
            expect(resp4).toBe(4);
            done();
        }), 20);
    });
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidMemoize = (0, memoize_1.memoize)(50);
            class T {
            }
            __decorate([
                nonValidMemoize,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@memoize is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should use provided cache', (done) => {
        const cache = new Map();
        class T {
            foo() {
                return this.goo();
            }
            goo() {
                return 1;
            }
        }
        __decorate([
            (0, memoize_1.memoize)({ expirationTimeMs: 30, cache }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Number)
        ], T.prototype, "foo", null);
        const spy = jest.spyOn(T.prototype, 'goo');
        const t = new T();
        t.foo();
        setTimeout(() => {
            t.foo();
            expect(spy).toHaveBeenCalledTimes(1);
            cache.delete('[]');
            t.foo();
            expect(spy).toHaveBeenCalledTimes(2);
            done();
        }, 10);
    });
    it('should verify memoize key foo as function', () => __awaiter(void 0, void 0, void 0, function* () {
        const mapper = jest.fn((x, y) => `${x}_${y}`);
        class T {
            fooWithMapper(x, y) {
                return this.goo(x, y);
            }
            goo(x, y) {
                return x + y;
            }
        }
        __decorate([
            (0, memoize_1.memoize)({ expirationTimeMs: 10, keyResolver: mapper }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String, String]),
            __metadata("design:returntype", String)
        ], T.prototype, "fooWithMapper", null);
        const t = new T();
        const spyFooWithMapper = jest.spyOn(T.prototype, 'goo');
        t.fooWithMapper('x', 'y');
        t.fooWithMapper('x', 'y');
        expect(mapper.mock.calls.length).toBe(2);
        expect(spyFooWithMapper).toHaveBeenCalledTimes(1);
        expect(spyFooWithMapper).toHaveBeenCalledWith('x', 'y');
    }));
    it('should verify memoize key foo as string - method name', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            foo(x, y) {
                return `${x}_${y}`;
            }
            fooWithInnerMapper(x, y) {
                return this.goo(x, y);
            }
            goo(x, y) {
                return x + y;
            }
        }
        __decorate([
            (0, memoize_1.memoize)({ expirationTimeMs: 10, keyResolver: 'foo' }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String, String]),
            __metadata("design:returntype", String)
        ], T.prototype, "fooWithInnerMapper", null);
        const t = new T();
        const spyFooWithMapper = jest.spyOn(T.prototype, 'goo');
        const spyMapper = jest.spyOn(T.prototype, 'foo');
        t.fooWithInnerMapper('x', 'y');
        t.fooWithInnerMapper('x', 'y');
        expect(spyMapper).toHaveBeenCalledTimes(2);
        expect(spyFooWithMapper).toHaveBeenCalledTimes(1);
        expect(spyFooWithMapper).toHaveBeenCalledWith('x', 'y');
    }));
    it('should verify that by default the cache is never cleaned', () => __awaiter(void 0, void 0, void 0, function* () {
        const cache = new Map();
        class T {
            foo() {
                return 1;
            }
        }
        __decorate([
            (0, memoize_1.memoize)({ cache }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Number)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo();
        expect(cache.size).toEqual(1);
        yield (0, test_utils_1.sleep)(50);
        expect(cache.size).toEqual(1);
    }));
});
//# sourceMappingURL=memoize.spec.js.map