"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const multi_dispatch_1 = require("./multi-dispatch");
const utils_1 = require("../common/utils/utils");
describe('retry', () => {
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidMultiDispatch = (0, multi_dispatch_1.multiDispatch)(50);
            class T {
            }
            __decorate([
                nonValidMultiDispatch,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@multiDispatch is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should dispatch twice and resolve', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.counter = 0;
            }
            foo() {
                this.counter += 1;
                if (this.counter === 1) {
                    return Promise.reject(new Error('no'));
                }
                return Promise.resolve('yes');
            }
        }
        __decorate([
            (0, multi_dispatch_1.multiDispatch)(2),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        const res = yield t.foo();
        expect(t.counter).toEqual(2);
        expect(res).toEqual('yes');
    }));
    it('should get last error if all rejected', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.counter = 0;
            }
            foo() {
                return __awaiter(this, void 0, void 0, function* () {
                    this.counter += 1;
                    if (this.counter === 1) {
                        yield (0, utils_1.sleep)(100);
                        return Promise.reject(new Error('slowest'));
                    }
                    yield (0, utils_1.sleep)(50);
                    return Promise.reject(new Error('fastest'));
                });
            }
        }
        __decorate([
            (0, multi_dispatch_1.multiDispatch)(2),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        try {
            yield t.foo();
            throw new Error('should not reach here');
        }
        catch (e) {
            expect(t.counter).toEqual(2);
            expect(e.message).toEqual('slowest');
        }
    }));
    it('should dispatch twice return faster', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.counter = 0;
            }
            foo() {
                return __awaiter(this, void 0, void 0, function* () {
                    this.counter += 1;
                    if (this.counter === 1) {
                        yield (0, utils_1.sleep)(100);
                        return Promise.resolve('slow');
                    }
                    yield (0, utils_1.sleep)(50);
                    return Promise.resolve('fast');
                });
            }
        }
        __decorate([
            (0, multi_dispatch_1.multiDispatch)(2),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        const res = yield t.foo();
        expect(t.counter).toEqual(2);
        expect(res).toEqual('fast');
    }));
});
//# sourceMappingURL=multi-dispatch.spec.js.map