"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const refreshable_1 = require("./refreshable");
const test_utils_1 = require("../common/test-utils");
describe('refreshable', () => {
    const originalSetInterval = global.setInterval;
    const unrefMock = jest.fn();
    function useFakeSetInterval() {
        global.setInterval = jest.fn(() => ({ unref: unrefMock }));
    }
    function restoreSetInterval() {
        global.setInterval = originalSetInterval;
    }
    afterEach(restoreSetInterval);
    it('should call unref on setInterval', () => __awaiter(void 0, void 0, void 0, function* () {
        useFakeSetInterval();
        const foo = (0, refreshable_1.refreshable)({
            dataProvider: () => Promise.resolve(0),
            intervalMs: 50,
        });
        const t = { prop: 0 };
        foo(t, 'prop');
        yield (0, test_utils_1.sleep)(10);
        expect(unrefMock).toBeCalled();
    }));
    it('should populate refreshable property', () => __awaiter(void 0, void 0, void 0, function* () {
        let fooCtr = 0;
        let gooCtr = 0;
        const foo = () => __awaiter(void 0, void 0, void 0, function* () {
            const tmp = gooCtr;
            fooCtr += 1;
            return tmp;
        });
        const goo = () => __awaiter(void 0, void 0, void 0, function* () {
            const tmp = gooCtr;
            gooCtr += 1;
            return tmp;
        });
        const fooDec = (0, refreshable_1.refreshable)({
            dataProvider: foo,
            intervalMs: 50,
        });
        const gooDec = (0, refreshable_1.refreshable)({
            dataProvider: goo,
            intervalMs: 50,
        });
        const t = { prop: 9, proop: 4 };
        fooDec(t, 'prop');
        gooDec(t, 'proop');
        yield (0, test_utils_1.sleep)(10);
        expect(t.prop).toBe(0);
        expect(t.proop).toBe(0);
        yield (0, test_utils_1.sleep)(60);
        expect(t.prop).toBe(1);
        expect(t.proop).toBe(1);
        t.prop = null;
        t.proop = 100;
        yield (0, test_utils_1.sleep)(50);
        expect(t.prop).toBe(1);
        expect(t.proop).toBe(2);
    }));
});
//# sourceMappingURL=refreshable.spec.js.map