"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("../common/test-utils");
const throttle_async_1 = require("./throttle-async");
describe('throttle-async', () => {
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonThrottleAsync = (0, throttle_async_1.throttleAsync)(50);
            class T {
            }
            __decorate([
                nonThrottleAsync,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@throttleAsync is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should verify method invocation is throttled 1', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.prop = 0;
            }
            foo(x) {
                return __awaiter(this, void 0, void 0, function* () {
                    this.prop += 1;
                    yield (0, test_utils_1.sleep)(30);
                    return x;
                });
            }
        }
        __decorate([
            (0, throttle_async_1.throttleAsync)(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        return new Promise((resolve) => __awaiter(void 0, void 0, void 0, function* () {
            const t = new T();
            expect(t.prop).toEqual(0);
            t.foo('a').then((res) => {
                expect(res).toEqual('a');
            });
            expect(t.prop).toEqual(1);
            t.foo('b').then((res) => {
                expect(res).toEqual('b');
                resolve(null);
            });
            expect(t.prop).toEqual(1);
            yield (0, test_utils_1.sleep)(20);
            expect(t.prop).toEqual(1);
            yield (0, test_utils_1.sleep)(50);
            expect(t.prop).toEqual(2);
        }));
    }));
    it('should verify method invocation is throttled 2', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.prop = 0;
            }
            foo() {
                return __awaiter(this, void 0, void 0, function* () {
                    this.prop += 1;
                    yield (0, test_utils_1.sleep)(20);
                    return this.prop;
                });
            }
        }
        __decorate([
            (0, throttle_async_1.throttleAsync)(2),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        return new Promise((resolve) => {
            const t = new T();
            t.foo().then((res) => {
                expect(res).toEqual(2);
                expect(t.prop).toEqual(2);
            });
            t.foo().then((res) => {
                expect(res).toEqual(2);
                expect(t.prop).toEqual(2);
                resolve(null);
            });
        });
    }));
    it('should work also with exceptions', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.prop = 0;
            }
            foo(x) {
                return __awaiter(this, void 0, void 0, function* () {
                    this.prop += 1;
                    yield (0, test_utils_1.sleep)(30);
                    if (this.prop === 1) {
                        throw new Error('blarg');
                    }
                    return x;
                });
            }
        }
        __decorate([
            (0, throttle_async_1.throttleAsync)(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        return new Promise((resolve) => {
            const t = new T();
            t.foo('a')
                .then(() => {
                throw new Error('should get to this point');
            })
                .catch((e) => {
                expect(e.message).toEqual('blarg');
            });
            t.foo('b')
                .then((res) => {
                expect(res).toEqual('b');
                resolve(null);
            });
        });
    }));
    it('should validate methods invoked in time', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.prop = 0;
            }
            foo(x) {
                return __awaiter(this, void 0, void 0, function* () {
                    this.prop += 1;
                    yield (0, test_utils_1.sleep)(20);
                    return x;
                });
            }
        }
        __decorate([
            (0, throttle_async_1.throttleAsync)(2),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo('a');
        expect(t.prop).toEqual(1);
        t.foo('b');
        expect(t.prop).toEqual(2);
        yield (0, test_utils_1.sleep)(30);
        t.foo('c');
        expect(t.prop).toEqual(3);
        const val = yield t.foo('d');
        expect(t.prop).toEqual(4);
        expect(val).toEqual('d');
    }));
    it('should validate methods invoked between times', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.prop = 0;
            }
            foo(x) {
                return __awaiter(this, void 0, void 0, function* () {
                    this.prop += 1;
                    yield (0, test_utils_1.sleep)(100);
                    return x;
                });
            }
        }
        __decorate([
            (0, throttle_async_1.throttleAsync)(2),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const start = new Date();
        const t = new T();
        t.foo('a');
        t.foo('b');
        t.foo('c');
        t.foo('d');
        t.foo('e');
        t.foo('f');
        t.foo('g');
        t.foo('h');
        t.foo('j');
        yield t.foo('k');
        const seconds = (new Date().getTime() - start.getTime()) / 1000;
        expect(seconds).toBeGreaterThanOrEqual(0.5);
        expect(seconds).toBeLessThan(0.6);
    }));
});
//# sourceMappingURL=throttle-async.spec.js.map