var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { onError } from './on-error';
describe('onError', () => {
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidOnError = onError({ func: null });
            class T {
            }
            __decorate([
                nonValidOnError,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@onError is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should verify onError called on exception, when as string', () => {
        class T {
            constructor() {
                this.prop = 3;
            }
            foo(x) {
                return this.goo(x);
            }
            goo(x) {
                throw new Error('arr');
            }
            onError(e, args) {
                expect(e.message).toBe('arr');
                expect(args).toEqual([1]);
                expect(this.prop).toBe(3);
            }
        }
        __decorate([
            onError({ func: 'onError' }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", Object)
        ], T.prototype, "foo", null);
        const t = new T();
        const spyGoo = jest.spyOn(T.prototype, 'goo');
        const spyAfter = jest.spyOn(T.prototype, 'onError');
        t.foo(1);
        expect(spyGoo).toBeCalledTimes(1);
        expect(spyGoo).toBeCalledWith(1);
        expect(spyAfter).toBeCalledTimes(1);
    });
    it('should verify onError called on exception, when as function', () => {
        const onErrorFunc = jest.fn((e, args) => {
            expect(e.message).toBe('arr');
            expect(args).toEqual([1]);
        });
        class T {
            foo(x) {
                return this.goo(x);
            }
            goo(x) {
                throw new Error('arr');
            }
        }
        __decorate([
            onError({ func: onErrorFunc }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", Object)
        ], T.prototype, "foo", null);
        const t = new T();
        const spyGoo = jest.spyOn(T.prototype, 'goo');
        t.foo(1);
        expect(spyGoo).toBeCalledTimes(1);
        expect(spyGoo).toBeCalledWith(1);
        expect(onErrorFunc).toBeCalledTimes(1);
    });
    it('should verify onError called on exception, when function is async', () => __awaiter(void 0, void 0, void 0, function* () {
        const onErrorFunc = jest.fn((e, args) => __awaiter(void 0, void 0, void 0, function* () {
            expect(e.message).toBe('error');
            expect(args).toEqual([1]);
        }));
        class T {
            foo(x) {
                return Promise.reject(new Error('error'));
            }
        }
        __decorate([
            onError({ func: onErrorFunc }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        yield t.foo(1);
        expect(onErrorFunc).toBeCalledTimes(1);
    }));
    it('should verify onError was not called when no error, and the function is async', () => __awaiter(void 0, void 0, void 0, function* () {
        const onErrorFunc = jest.fn(() => __awaiter(void 0, void 0, void 0, function* () {
        }));
        class T {
            foo() {
                return Promise.resolve();
            }
        }
        __decorate([
            onError({ func: onErrorFunc }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        yield t.foo();
        expect(onErrorFunc).not.toBeCalled();
    }));
    it('should verify onError called on exception, when function is sync', () => {
        const onErrorFunc = jest.fn((e, args) => __awaiter(void 0, void 0, void 0, function* () {
            expect(e.message).toBe('arr');
            expect(args).toEqual([1]);
        }));
        class T {
            foo(x) {
                throw new Error('arr');
            }
        }
        __decorate([
            onError({ func: onErrorFunc }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number]),
            __metadata("design:returntype", Object)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo(1);
        expect(onErrorFunc).toBeCalledTimes(1);
    });
});
//# sourceMappingURL=on-error.spec.js.map